@@ -2,10 +2,9 @@
 class Keys:
     PIPELINE = 'pipeline'
     DATASET_DEFINITION = 'dataset_definition'
-    QC_TESTS = 'qc_tests'
-    QC_TESTS_COORD = 'coordinate_variable_qc_tests'
+    DEFAULTS = 'variable_defaults'
+    QC_TESTS = 'quality_management'
     ATTRIBUTES = 'attributes'
     DIMENSIONS = 'dimensions'
     VARIABLES = 'variables'
     ALL = 'ALL'
-    
\ No newline at end of file
