@@ -4,6 +4,8 @@ import xarray as xr
 
 class VARS(object):
     ALL = 'ALL'
+    COORDS = 'COORDS'
+    DATA_VARS = 'DATA_VARS'
 
 
 class ATTS(object):
@@ -22,7 +24,7 @@ class ATTS(object):
     REFERENCES = 'references'
     INPUT_FILES = 'input_files'
     LOCATION_ID = 'location_id'
-    DATASTREAM = 'datastream'
+    DATASTREAM = 'datastream_name'
     DATA_LEVEL = 'data_level'
     LOCATION_DESCRPTION = 'location_description'
     INSTRUMENT_NAME = 'instrument_name'
@@ -39,4 +41,4 @@ class ATTS(object):
     FAIL_RANGE = 'fail_range'
     WARN_RANGE = 'warn_range'
     FILL_VALUE = '_FillValue'
-
+    CORRECTIONS_APPLIED = 'corrections_applied'
