@@ -3,13 +3,11 @@ import xarray as xr
 from tsdat.config import Config
 
 from .file_handlers import AbstractFileHandler
-from .file_handlers import register_filehandler
 
 
-@register_filehandler([".nc", ".cdf"])
 class NetCdfHandler(AbstractFileHandler):
-    @staticmethod
-    def write(ds: xr.Dataset, filename: str, config: Config = None, **kwargs) -> None:
+
+    def write(self, ds: xr.Dataset, filename: str, config: Config = None, **kwargs) -> None:
         # We have to make sure that time variables do not have units set as attrs,
         # and instead have units set on the encoding or else xarray will crash
         # when trying to save:
@@ -24,10 +22,7 @@ class NetCdfHandler(AbstractFileHandler):
 
         ds.to_netcdf(filename, format='NETCDF4')
 
-    @staticmethod
-    def read(filename: str, config: Config = None, **kwargs) -> xr.Dataset:
-        # TODO: need to have xr.Dataset close the file automatically if user
-        #  uses "with" - add a resource manager api
-        # ds_disk = xr.open_dataset(filename)
-        # TODO: Use config?
-        return xr.open_dataset(filename)
\ No newline at end of file
+    def read(self, filename: str, config: Config = None, **kwargs) -> xr.Dataset:
+        # We are using xr.load_dataset because it will load the whole file
+        # into memory and close the underlying file handle.
+        return xr.load_dataset(filename)
